#version 430 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
} fs_in;

struct Light {
    vec3 Position;
    vec3 Color;

    vec3 ambient;
    vec3 diffuse;
    vec3 specular;

    float constant;
    float linear;
    float quadratic;
};

struct Material {
    sampler2D diffuse;
    sampler2D specular;    
    float shininess;
}; 

uniform Light lights[6];
uniform sampler2D diffuseTexture;
uniform vec3 viewPos;

uniform Material material;
uniform float Lightmap;
uniform sampler2D lightmap;

void main()
{   
    vec3 result; 
 
    if(Lightmap==1) {
    
        vec3 color = texture(diffuseTexture, fs_in.TexCoords).rgb;
        vec3 norm = normalize(fs_in.Normal);
        // ambient
        vec3 ambient = lights[4].ambient * color;

        // lighting
        // vec3 lighting = vec3(0.0);
		for(int i = 4; i < 6; i++){
  
            // diffuse
            vec3 lightDir = normalize(lights[i].Position - fs_in.FragPos);
            float diff = max(dot(lightDir, norm), 0.0);
            vec3 diffuse = lights[i].diffuse * diff * color; 
            
            // specular
            vec3 viewDir = normalize(viewPos - fs_in.FragPos);
            vec3 reflectDir = reflect(-lightDir, norm);  
            float spec = pow(max(dot(viewDir, reflectDir), 0.0), 50.0f); // material.shininess geht ned statt 50
            vec3 specular = lights[i].specular * spec * color;  

            // attenuation (use quadratic as we have gamma correction)
            float distance = length(lights[i].Position - fs_in.FragPos);
            float attenuation = 1.0 / (lights[i].constant + lights[i].linear * distance + lights[i].quadratic * (distance * distance));  

            ambient  *= attenuation; 
	        diffuse   *= attenuation;
            specular *=attenuation;
			
			if(i == 4){
				result += ((ambient + diffuse + specular) * vec3(0.3, 1.0, 1.0)); 	//color of the laser_ray
			}
            else{
				result += ((ambient + diffuse + specular) * vec3(0.5, 0.0, 0.0));
			}
		}
		
		// check whether result is higher than some threshold, if so, output as bloom threshold color
         float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722));
         if(brightness > 0.8){
			BrightColor = vec4(result, 1.0);
		 }
         else
             BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
      
         FragColor = vec4(result, 1.0);
         FragColor += texture(material.diffuse, fs_in.TexCoords) * (texture(lightmap, fs_in.TexCoords)*2.0);
    }


    else {
         vec3 color = texture(diffuseTexture, fs_in.TexCoords).rgb;
         vec3 norm = normalize(fs_in.Normal);
         // ambient
         vec3 ambient = lights[0].ambient * color;

         // lighting
        // vec3 lighting = vec3(0.0);
        
         for(int i = 0; i < 6; i++)
         {
             // diffuse
             vec3 lightDir = normalize(lights[i].Position - fs_in.FragPos);
             float diff = max(dot(lightDir, norm), 0.0);
             vec3 diffuse = lights[i].Color * diff * color; 
             
             // specular
             vec3 viewDir = normalize(viewPos - fs_in.FragPos);
             vec3 reflectDir = reflect(-lightDir, norm);  
             float spec = pow(max(dot(viewDir, reflectDir), 0.0), 50.0f); // material.shininess geht ned statt 50
             vec3 specular = lights[i].specular * spec * color;  

             // attenuation (use quadratic as we have gamma correction)
             float distance = length(lights[i].Position - fs_in.FragPos);
             float attenuation = 1.0 / (lights[i].constant + lights[i].linear * distance + lights[i].quadratic * (distance * distance));  

             ambient  *= attenuation; 
	         diffuse   *= attenuation;
             specular *=attenuation;

             if(i == 4){
	        			result += ((ambient + diffuse + specular) * vec3(0.3, 1.0, 1.0)); 	//color of the laser_ray
	        		}		
	        		else{
	        			result += ambient + diffuse + specular;
	        		}
             
                     
         }
           
         // check whether result is higher than some threshold, if so, output as bloom threshold color
         float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722));
         if(brightness > 0.5){
			BrightColor = vec4(result, 1.0);
		 }
         else
             BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
             FragColor = vec4(result, 1.0);
         
         }
    

}